@@ -223,6 +223,30 @@ def test_ec_public_numbers_repr():
     assert repr(pn) == "<EllipticCurvePublicNumbers(curve=secp256r1, x=2, y=3>"
 
 
+def test_ec_public_numbers_hash():
+    pn1 = ec.EllipticCurvePublicNumbers(2, 3, ec.SECP256R1())
+    pn2 = ec.EllipticCurvePublicNumbers(2, 3, ec.SECP256R1())
+    pn3 = ec.EllipticCurvePublicNumbers(1, 3, ec.SECP256R1())
+
+    assert hash(pn1) == hash(pn2)
+    assert hash(pn1) != hash(pn3)
+
+
+def test_ec_private_numbers_hash():
+    numbers1 = ec.EllipticCurvePrivateNumbers(
+        1, ec.EllipticCurvePublicNumbers(2, 3, DummyCurve())
+    )
+    numbers2 = ec.EllipticCurvePrivateNumbers(
+        1, ec.EllipticCurvePublicNumbers(2, 3, DummyCurve())
+    )
+    numbers3 = ec.EllipticCurvePrivateNumbers(
+        2, ec.EllipticCurvePublicNumbers(2, 3, DummyCurve())
+    )
+
+    assert hash(numbers1) == hash(numbers2)
+    assert hash(numbers1) != hash(numbers3)
+
+
 @pytest.mark.requires_backend_interface(interface=EllipticCurveBackend)
 class TestECWithNumbers(object):
     @pytest.mark.parametrize(
@@ -791,6 +815,34 @@ class TestEllipticCurvePEMPublicKeySerialization(object):
         )
         assert serialized == key_bytes
 
+    def test_public_bytes_openssh(self, backend):
+        _skip_curve_unsupported(backend, ec.SECP192R1())
+        _skip_curve_unsupported(backend, ec.SECP256R1())
+
+        key_bytes = load_vectors_from_file(
+            os.path.join(
+                "asymmetric", "PEM_Serialization", "ec_public_key.pem"
+            ),
+            lambda pemfile: pemfile.read(), mode="rb"
+        )
+        key = serialization.load_pem_public_key(key_bytes, backend)
+
+        ssh_bytes = key.public_bytes(
+            serialization.Encoding.OpenSSH, serialization.PublicFormat.OpenSSH
+        )
+        assert ssh_bytes == (
+            b"ecdsa-sha2-nistp256 AAAAE2VjZHNhLXNoYTItbmlzdHAyNTYAAAAIbmlzdHAy"
+            b"NTYAAABBBCS8827s9rUZyxZTi/um01+oIlWrwLHOjQxRU9CDAndom00zVAw5BRrI"
+            b"KtHB+SWD4P+sVJTARSq1mHt8kOIWrPc="
+        )
+
+        key = ec.generate_private_key(ec.SECP192R1(), backend).public_key()
+        with pytest.raises(ValueError):
+            key.public_bytes(
+                serialization.Encoding.OpenSSH,
+                serialization.PublicFormat.OpenSSH
+            )
+
     def test_public_bytes_invalid_encoding(self, backend):
         _skip_curve_unsupported(backend, ec.SECP256R1())
         key = load_vectors_from_file(
